/**
 * Module dependencies.
 */

var express = require('express');
var pg = require('pg');
var query = require('pg-query');

var app = module.exports = express.createServer();

var filtresGlobals = {};

query.connectionParameters = 'tcp://dani:dani@localhost/postgres';

// Configuration

app.configure(function(){
    app.set('views', __dirname + '/views');
    app.use(express.bodyParser());
    app.use(express.methodOverride());
    app.use(app.router);
    app.use(express.static(__dirname + '/public'));
});

app.configure('development', function(){
    app.use(express.errorHandler({ dumpExceptions: true, showStack: true })); 
});

app.configure('production', function(){
    app.use(express.errorHandler()); 
});

app.set('view options', {
    layout: false
});

app.register('.html', {
    compile: function(str, options){
        return function(locals){
            return str;
        };
    }
  });

// Routes
app.get('/', function(req, res){
    var d = new Date();
    console.log('New connection: ' + d.toLocaleString() + ' - ' + req.connection.remoteAddress);
    res.render('index.html', { });
});

app.post('/getInitialLayer', function(req, res){
    getInitialLayer(res);
});

app.post('/getLayers', function(req, res){
    res.send(filtresGlobals);
});

app.post('/processFilter', function(req, res){
    processFilter(req, res);
});

app.post('/getDataLayerSideBar', function(req, res) {
    getDataLayerSideBar(req, res);
});

app.post('/getDataLayer', function(req, res) {
    getDataLayer(req, res, '');
});

function getDataLayer(req, res, condition) {
    var selectedLayer = filtresGlobals[req.body.selectedLayer];

    var sql = selectedLayer.consultaSQL.replace(/\$CONDITION\$/g, condition);
    sql = sql.replace(/\$SELECTOR_TERRITORIAL\$/g, 'codi_rs_residencia');
    sql = sql.replace(/\$DIVISOR_TERRITORIAL\$/g, 'regio_sanitaria');

    query(sql, '', function(err, rows, result) {
        var featureCollection = new Array();

        if (result.rows.length == 0) { 
            res.statusCode = 401;
            res.send('No s\'han trobat resultats.');
            return;
        }

        for (i = 0; i < result.rows.length; i++)
        {
            var a = new myCollection();
            a.camp = result.rows[i].camp;
            a.valor = result.rows[i].valor;
            a.divisorTerritorial = result.rows[i].nom;
            a.valorTotal = result.rows[i].valortotal;
            a.absolutTotal = result.rows[i].absoluttotal;
            featureCollection[result.rows[i].id] = a;
        }

        res.send(featureCollection);
    });
}

function getDataLayerSideBar(req, res) {
    var selectedLayer = filtresGlobals[req.body.selectedLayer];

    var html = selectedLayer.getDataLayerSideBar();

    res.send(html);
}

function getInitialLayer(res) {

    var sql = 'SELECT row_to_json(fc)::text as shape FROM ( SELECT \'FeatureCollection\' As type, array_to_json(array_agg(f)) As features FROM (SELECT \'Feature\' As type, lg.codi_regio_sanitaria as id, ST_AsGeoJSON(ST_Transform(lg.geom,4326))::json As geometry FROM regions_sanitaries_25831 As lg   ) As f )  As fc;';

    query(sql, '', function(err, rows, result) {
        res.send(JSON.parse(result.rows[0].shape));
    });
}

function processFilter(req, res) {
    var selectedLayer = filtresGlobals[req.body.selectedLayer];
    var condition = '';
    var str = "'";

    for (var keyFiltre in selectedLayer.filtres) {
        var filtre = selectedLayer.filtres[keyFiltre];

        switch(filtre.tipus) {
            case 'checkbox':
                var valorsSel = req.body[keyFiltre];

                if(!valorsSel || valorsSel.length == 0) {
                   res.status(500).send('És necessari seleccionar com a mínim un valor per al filtre ' + (filtre.alias || keyFiltre));  return;
                }

                if (filtre.valors != null && filtre.valors.length != valorsSel.length) {
                    condition = condition + ' AND (';
                    for (var i = 0; i < valorsSel.length; i++) {
                        if (i != 0) condition = condition + ' OR ';
                            condition = condition + filtre.camp + ' = ' + str + valorsSel[i] + str;
                    }
                    condition = condition + ')';
                }
                break;
            case 'slider':
                if (filtre.valors[0] != req.body['minValue'] || filtre.valors[1] != req.body['maxValue'] ) {
                    condition = condition + ' AND ' + filtre.camp + ' BETWEEN ' + req.body['minValue'] + ' AND ' + req.body['maxValue'];
                }
                break;
            default:
                console.log('type not supported');
        }
    }
    getDataLayer(req, res, condition);
}

function getHTMLCheckBox(filtre) {
    var html = '';

    html = html + '<label for="sua' + filtre.camp + '" class="checkbox">';
    html = html + '<input type="checkbox" checked="checked" class="checkbox" id="sua' + filtre.camp + '"/>Selecciona/desselecciona tot</label>';
    html = html + '<br>';

    for(var i = 0; i < filtre.valors.length; i++) {
        var valor = filtre.valors[i];
        var texte = filtre.textes[i] || valor;
        html = html + '';
        html = html + '<label for="' + filtre.camp + '_' + valor + '" class="checkbox">';
        html = html + '<input type="checkbox" checked="checked" class="checkbox" id="' + filtre.camp + '_' + valor + '" name="' + filtre.camp + '[]" value="' + valor + '" />' + texte + '</label>';
        html = html + '<br>';
    }

    html = html + '<script>';
    html = html + '$("#sua' + filtre.camp + '").change(function () {' +    
                      '$divFilter = $(this).closest(".filter");' + 
                      'var checkboxes = $divFilter.find(":checkbox");' + 
                      'if($(this).is(":checked")) {' +
                          'checkboxes.prop("checked", true);' +
                      '} else {' +
                          'checkboxes.prop("checked", false);' +
                      '}' +
                  '});'
    html = html + '</script>';

    return html;
}

function getHTMLRangeSlider(filtre) {
    var html = '<div id="slider" style="margin-left: 15px; margin-right: 15px;"></div><input type="text" id="minValue" name="minValue" value="' + filtre.valors[0] + '" size="1" style="text-align: center; border: none; background-color: transparent; margin-top: 5px;" readonly /><input type="text" id="maxValue" name="maxValue" value="' + filtre.valors[1] + '" size="1" style="text-align: center; margin-left: 324px; border: none; background-color: transparent; margin-top: 5px;" readonly /><br><script>$(function() { var minValue = $("#minValue"); var maxValue = $("#maxValue"); $("#slider").slider({range: true, max:' + filtre.valors[1] + ', min:' + filtre.valors[0] + ', values: [ ' + filtre.valors[0] + ', ' + filtre.valors[1] + ' ], slide: function(event, ui) { minValue.val(ui.values[0]); maxValue.val(ui.values[1]); } }); }); </script>';
    
    return html;
}

function getHTMLLocalitzacio(filtre) {
    var html = '<div id="localitzacio" style="margin-left: 15px; margin-right: 15px;"></div><input type="text" id="cadenaLocalitzacio" name="cadenaLocalitzacio" style="width:100%" /><br>';
    
    return html;
}

function loadLayers() {
    console.log('loadLayers init');
    
    var sql = 'select l.nom, l.descripcio, l.consultasql, l.taula, lf.camp, lf.tipus, lf.alias, lf.consulta from layer l LEFT JOIN layerfilter lf ON l.nom = lf.nom_layer order by l.nom, lf.camp';

    query(sql, '', function(err, rows, result) {
        for (h = 0; h < rows.length; h++) {
            var layerFilters = {};
            var layer;

            if (filtresGlobals[rows[h].nom] != null) {
                layer = filtresGlobals[rows[h].nom];
                layerFilters = layer.filtres;
                var layerFilter = new myLayerFilter(rows[h].camp, rows[h].tipus, new Array(), new Array(), rows[h].consulta, rows[h].alias);
                layerFilters[rows[h].camp] = layerFilter;
                layer.filtres = layerFilters;
            } else {
                var layerFilter = {};
                if(rows[h].camp != null) {
                    layerFilter = new myLayerFilter(rows[h].camp, rows[h].tipus, new Array(), new Array(), rows[h].consulta, rows[h].alias);
                    layerFilters[rows[h].camp] = layerFilter;
                }
                
                layer = new myLayer(rows[h].nom, rows[h].descripcio, rows[h].consultasql, rows[h].taula, layerFilters);

                filtresGlobals[rows[h].nom] = layer;
            }

            getFilterValues(layer);
        } 
    });

    console.log('loadLayers end');
}

function getFilterValues(selectedLayer) {
    var sqlLayerFilterValues;
    var valors = new Array();

    for (var keyFiltre in selectedLayer.filtres) {
        var filtre = selectedLayer.filtres[keyFiltre];

        switch(filtre.tipus) {
            case 'checkbox':
                sqlLayerFilterValues = 'select distinct(' + filtre.camp + ') as values, \'' + keyFiltre + '\' as keyfiltre from ' + selectedLayer.taula + ' order by values';
                break;
            case 'slider':
                sqlLayerFilterValues = 'select min(' + filtre.camp + ') as minim, max(' + filtre.camp + ') as maxim, \'' + keyFiltre + '\' as keyfiltre from ' + selectedLayer.taula;
                break;
            default:
                console.log('type not supported');
        }

        if (filtre.consulta !== null) {
            sqlLayerFilterValues = filtre.consulta.replace('$keyfiltre$', keyFiltre);
        }

        query(sqlLayerFilterValues, '', function(err2, rowsLayerFilterValues, resultLayerFilterValues) {
            valors = new Array();
            textes = new Array();
            for (k = 0; k < rowsLayerFilterValues.length; k++) {
                var filtre = selectedLayer.filtres[rowsLayerFilterValues[k].keyfiltre];

                switch(filtre.tipus) {
                    case 'checkbox':
                        valors[k] = rowsLayerFilterValues[k].values;
                        textes[k] = rowsLayerFilterValues[k].texto;
                        break;
                    case 'slider':
                        valors[0] = rowsLayerFilterValues[k].minim;
                        valors[1] = rowsLayerFilterValues[k].maxim;
                        textes[0] = rowsLayerFilterValues[k].minim;
                        textes[1] = rowsLayerFilterValues[k].maxim;
                        break;
                    default:
                        console.log('type not supported');
                }
            filtre.valors = valors;
            filtre.textes = textes;
            }
        });
    }
}

// GeoJSON Feature Collection
function FeatureCollection(){
    this.type = 'FeatureCollection';
    this.features = new Array();
}

function myCollection(){
    this.camp = new String();
    this.valor = new String();
    this.divisorTerritorial = new String();
    this.valorTotal = new String();
    this.absolutTotal = new String();
}

var myLayer = function(nom, descripcio, consultaSQL, taula, filtres) {
    this.nom = nom;
    this.descripcio = descripcio;
    this.consultaSQL = consultaSQL;
    this.taula = taula;
    this.filtres = filtres;
};

var myLayerFilter = function(camp, tipus, valors, textes, consulta, alias) {
    this.camp = camp;
    this.tipus = tipus;
    this.valors = valors;
    this.textes = textes;
    this.consulta = consulta;
    this.alias = alias;
};

myLayer.prototype.getDataLayerSideBar = function() {
    var html = '';

    html = html + '<h1>Descripció</h1>';
    html = html + '<h4>' + this.descripcio + '</h4>';

    html = html + '<h1>Filtres</h1>';

    var filtres = this.filtres;

    if(Object.keys(filtres).length > 0) {
        html = html + '<form id="d-form" name="d-form"><input type="hidden" id="selectedLayer" name="selectedLayer" value="' + this.nom + '">';
    } else {
        html = html + '<h4>No es possible filtrar per cap camp</h4>';
    }

    for (var keyFiltre in filtres) {
        var filtre = filtres[keyFiltre];
        var titolFiltre = filtre.alias || filtre.camp;
        
        switch(filtre.tipus) {
            case 'checkbox':
                html = html + '<div class="filterTitle"><h4><i class="fa fa-plus-square-o"></i> ' + titolFiltre + '</h4></div>';
                html = html + '<div class="filter collapsed" id="' + titolFiltre + '" name="' + titolFiltre + '">';
                html = html + getHTMLCheckBox(filtre);
                html = html + '</div>'
                break;
            case 'slider':
                html = html + '<h4>' + titolFiltre + '</h4>';
                html = html + getHTMLRangeSlider(filtre);
                break;
            case 'localitzacio':
                html = html + '<h4>' + titolFiltre + '</h4>';
                html = html + getHTMLLocalitzacio(filtre);
                break;
            default:
                console.log('type not supported');
        }
    }

    html = html + '<script>';
    html = html + '$(".filterTitle").click(function () {' +    
                      '$filtre = $(this).next(".filter");' + 
                      'if($filtre.css("display") == "none") {' +
                          '$filtre.show(500);' +
                          '$(this).find(".fa").removeClass("fa fa-plus-square-o").addClass("fa fa-minus-square-o");' +
                      '} else {' +
                          '$filtre.hide(500);' +
                          '$(this).find(".fa").removeClass("fa fa-minus-square-o").addClass("fa fa-plus-square-o");' +
                      '}' +
                  '});'
    html = html + '</script>';

    if(Object.keys(filtres).length > 0) {
        html = html + '<br><input type="submit" value"Submit"></form><br>';
    }

    return html;
};

app.listen(3001);
console.log("Express server listening on port %d in %s mode", app.address().port, app.settings.env);
loadLayers();
