var map;
var dataLayer;
var specialLayer;
var specialData;
var specialPopup;
var centresSanitarisData;
var farmaciesData;
var data;
var capes;
var svg;
var g;
var legend;
var marcador;
var globalSelectedLayer;
var arrLegendGrades = [0, 5, 10, 15, 20, 25, 30, 35, 40, 45];


var sidebar = L.control.sidebar('sidebar');

var layerSelector = L.Control.extend({
    initialize: function (pLayer, options) {
        gLayer = pLayer;
        L.Util.setOptions(this, options);
    },
    onAdd: function (map) {
        var div = L.DomUtil.create('div', 'info layerSelector');
        div.innerHTML += '<h4>Selector de capes</h4>';
        for (var keyLayer in gLayer) { 
            div.innerHTML += '<input type="radio" id="' + keyLayer + '" name="selectedLayer" value="' + keyLayer + '">';
            div.innerHTML += '<label for="' + keyLayer + '">' + keyLayer.replace('$$','') + '</label><br>';
        }

        return div;
    }            
});

$(document).ready(function () {
    //map = new L.map('map', {zoom: 13, maxZoom: 9, minZoom: 8}).setView([41.69676, 1.88042],8);
    map = new L.map('map', {zoom: 13}).setView([41.69676, 1.88042],8);  

    //Capa Base
    var osm = new L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {attribution: '© <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'}).addTo(map); 

    getInitialLayer();

    legend = L.control({position: 'bottomright'});

    legend.onAdd = function (map) {
        var div = L.DomUtil.create('div', 'info legend'),
        labels = [];

        // loop through our density intervals and generate a label with a colored square for each interval
        for (var i = 0; i < arrLegendGrades.length; i++) {
            div.innerHTML += '<i style="background:' + getColor(arrLegendGrades[i] + 1) + '"></i> ' + arrLegendGrades[i] + (arrLegendGrades[i + 1] ? '&ndash;' + arrLegendGrades[i + 1] + '%<br>' : '+%');
        }
        
        return div;
    };
    legend.addTo(map);
});

function processFilter(e) {
    $.ajax({
        type: 'POST',
        url: '/processFilter',
        dataType: 'json',
        contentType: 'application/x-www-form-urlencoded',
        data: $(this).serialize(),
        success: function(data) {
            drawDataLayer(data);
        },
        error: function(jqXhr, textStatus, errorThrown) {
            alert("Error: " + jqXhr.responseText);
        }
     });

    e.preventDefault();
}

function processSpecialFilter(e) {
    clearLayers();

    e.preventDefault();

    var adreca = $("#cadenaLocalitzacio").val();

    if (adreca != undefined) {
        $.ajax({
            type: 'GET',
            url: 'http://maps.googleapis.com/maps/api/geocode/json',
            dataType: 'json',
            data: {address : adreca},
            success: function (result) {
                if (result.status == 'ZERO_RESULTS') {
                    if (adreca != '') {
                        alert("No s'ha pogut trobar l'adreça. Impossible calcular la distancia als centres.");
                    }
                } else {
                    var lat = result.results[0].geometry.location.lat;
                    var long = result.results[0].geometry.location.lng;
                    marcador = L.marker([lat, long]).addTo(map);
                    marcador.bindPopup(result.results[0].formatted_address);
                    map.setZoomAround([lat, long], 14);
                }
                specialLayer = L.geoJson(specialData, {
                    pointToLayer: function(feature, latlng) {
                        return new L.circleMarker(latlng, {radius: 3, color: 'red', fillColor: 'red', fillOpacity: 1 });
                    },
                    onEachFeature: function(feature, layer) {
                        if (feature.properties) {
                            var distancia = -1;
                            if (marcador){
                                distancia = marcador.getLatLng().distanceTo(layer.getLatLng());
                            }
                            feature.properties.distancia = distancia;
                        }
                    },
                    filter: function(feature, layer) {
                        for (var keyFiltre in capes[globalSelectedLayer].filtres) {
                            var filtre = capes[globalSelectedLayer].filtres[keyFiltre];
                            var resultatFiltre = false;
                            switch(filtre.tipus) {
                                case 'checkbox':
                                    var bb = $("input[name='" + keyFiltre + "[]']:checkbox:checked");
                                    for (var i=0; i < bb.length; i++) {
                                       if (feature.properties[keyFiltre] == bb[i].value) {
                                            resultatFiltre = true;
                                        }
                                    }
                                    break;
                                case 'localitzacio':
                                    resultatFiltre = true;
                                    break;
                                default:
                                    resultatFiltre = true;
                            }

                            if (!resultatFiltre) return false;
                        }
                        return true;
                    }
                }).addTo(map);
                
                drawCentresLayer();
            },    
            error: function (req, status, error) {
                alert('Unable to get data from server');
            }
        });
    } 
}

function getDataLayer(selectedLayer) {
    $.ajax({
        type: 'POST',
        url: '/getDataLayer',
        dataType: 'json',
        data: {selectedLayer : selectedLayer},
        success: function (result) {
            drawDataLayer(result);
            getDataLayerSideBar(selectedLayer);
        },
        error: function (req, status, error) {
            alert('Unable to get data from server');
        }
    });
}

function drawSpecialLayers(selectedLayer) {
    globalSelectedLayer = selectedLayer;

    getDataLayerSideBar(selectedLayer);

    clearLayers();

    if (selectedLayer == '$$Centres sanitaris') {
        specialData = centresSanitarisData;
    } else if (selectedLayer == '$$Farmàcies') {
        specialData = farmaciesData;
    }

    specialLayer = L.geoJson(specialData, {
        pointToLayer: function(feature, latlng) {
            return new L.circleMarker(latlng, {radius: 3, color: 'red', fillColor: 'red', fillOpacity: 1 });
        },
    }).addTo(map);


    drawCentresLayer();
}

function drawCentresLayer() {
    svg = d3.select(map.getPanes().overlayPane).append("svg");
    g = svg.append("g").attr("class", "leaflet-zoom-hide");
    map.on("viewreset moveend", reset);


    reset();
}

// Use Leaflet to implement a D3 geometric transformation.
function projectPoint(x, y) {
    var point = map.latLngToLayerPoint(new L.LatLng(y, x));
    this.stream.point(point.x, point.y);
}

function reset() {
    var voronoiData = specialLayer.toGeoJSON();

    g.selectAll("path").remove();

    var voronoi = d3.geom.voronoi()
        .x(function(d) { return d.properties.x; })
        .y(function(d) { return d.properties.y; });

    //  create a d3.geo.path to convert GeoJSON to SVG
    var transform = d3.geo.transform({point: projectPoint}),
    path = d3.geo.path().projection(transform);

    var buildPathFromPoint = function(po) {
        return "M" + po.point.properties.cell.join("L") + "Z";
    }

    voronoiData.features.forEach(function(d) {
        var latlng = new L.LatLng(d.geometry.coordinates[1], d.geometry.coordinates[0]);
        var point = map.latLngToLayerPoint(latlng);

        d.properties.x = point.x;
        d.properties.y = point.y;
    });

    var fromVoronoi = [];
    voronoi(voronoiData.features).forEach(function(d) { 
        d.point.properties.cell = d;
        fromVoronoi.push(d);
    });

    var d4_features = g.selectAll("path")
        .data(fromVoronoi)
        .enter()
        .append("path")
        .attr("d", buildPathFromPoint);

    bounds = path.bounds(specialData);

    bounds[0][0] = bounds[0][0] - 10;
    bounds[0][1] = bounds[0][1] - 10;
    bounds[1][0] = bounds[1][0] + 10;
    bounds[1][1] = bounds[1][1] + 10;

    var topLeft = bounds[0];
    var bottomRight = bounds[1];

    svg .attr("width", bottomRight[0] - topLeft[0])
        .attr("height", bottomRight[1] - topLeft[1])
        .style("left", topLeft[0] + "px")
        .style("top", topLeft[1] + "px");

    g .attr("transform", "translate(" + -topLeft[0] + "," + -topLeft[1] + ")");

    d4_features.on("click",function(d){
        var ax = d.point.geometry.coordinates[0];
        var ay = d.point.geometry.coordinates[1];

        specialPopup = L.popup({closeOnClick:false}).setLatLng(new L.LatLng(ay, ax))
            .setContent(getMarkerText(d.point.properties))
            .openOn(map);
    });

    d4_features.attr("d", buildPathFromPoint)
        .attr("class", "point-cell")
        .attr("stroke", 'black')
        .attr('fill','none');
}                 

function getDataLayerSideBar(selectedLayer) {
    $.ajax({
        type: 'POST',
        url: '/getDataLayerSideBar',
        dataType: 'html',
        data: {selectedLayer : selectedLayer},
        success: function (result) {
            $("#toReplace").replaceWith('<div id="toReplace">' + result + '</div>');
            if (selectedLayer.startsWith("$$")) {
                 $('#d-form').submit(processSpecialFilter);
            } else {
                 $('#d-form').submit(processFilter);
            }
        },
        error: function (req, status, error) {
            alert('Unable to get data from server');
        }
    });
}

function getInitialLayer() {
    $.ajax({
        type: 'POST',
        url: '/getInitialLayer',
        dataType: 'json',
        contentType: 'application/json; charset=utf-8',
        success: function (result) {
            drawInitialLayer(result);
            sidebar.addTo(map);
            drawLayerSelector();
        },
        error: function (req, status, error) {
            alert('Unable to get data from server');
        }
    });

    var owsrootUrl = 'http://192.168.1.65:8080/geoserver/ows';
    //var owsrootUrl = 'http://cz5439.myfoscam.org:8080/geoserver/ows';

    var defaultParameters = {
        service : 'WFS',
        version : '2.0.0',
        request : 'GetFeature',
        typeName : 'Test1:centres-sanitaris',
        outputFormat : 'text/javascript',
        format_options : 'callback:getJson',
        SrsName : 'EPSG:4326'
    };

    var parameters = L.Util.extend(defaultParameters);
    var URL = owsrootUrl + L.Util.getParamString(parameters);

    $.ajax({
        url : URL,
        dataType : 'jsonp',
        jsonpCallback : 'getJson',
        success : function (result) {
            centresSanitarisData = result;
            continueLoading();
        }
    });

    function continueLoading(){
        defaultParameters = {
            service : 'WFS',
            version : '2.0.0',
            request : 'GetFeature',
            typeName : 'Test1:farmacies',
            outputFormat : 'text/javascript',
            format_options : 'callback:getJson',
            SrsName : 'EPSG:4326'
        };

        parameters = L.Util.extend(defaultParameters);
        URL = owsrootUrl + L.Util.getParamString(parameters);

        $.ajax({
            url : URL,
            dataType : 'jsonp',
            jsonpCallback : 'getJson',
            success : function (result) {    
                farmaciesData = result;
            }
        });
    }
}

function drawDataLayer(result) {
    clearLayers();

    dataLayer = L.geoJson(data, {
        onEachFeature: function (feature, layer) {
          var percent = ((result[feature.id].valor / result[feature.id].valorTotal) * 100);
          var absolutTotal = result[feature.id].absolutTotal;
          var valorTotal = result[feature.id].valorTotal;
          var valor = result[feature.id].valor;
          var content = '<h4>' + result[feature.id].divisorTerritorial + '</h4>';
          content += '<h5>' + result[feature.id].camp + ' totals a Catalunya: ' + numeral(absolutTotal).format('0,0') + '</h5>';
          content += '<h5>' + result[feature.id].camp + ' total: ' + numeral(valorTotal).format('0,0') + '</h5>';
          content += '<h5>' + result[feature.id].camp + ': ' + numeral(valor).format('0,0') + '</h5>';
          content += '<h5>Percentatge respecte al total: ' + numeral(percent).format('0,0.00') + '%</h5>';

          layer.setStyle(style(percent));
          var myPopUp = L.popup().setContent(content);
          layer.bindPopup(myPopUp);
        }
    }).addTo(map);
}

function style(value) {
    return {
        fillColor: getColor(value),
        weight: 2,
        opacity: 1,
        color: 'white',
        dashArray: '3',
        fillOpacity: 0.7
    };
}

function getColor(d) {
    return d > arrLegendGrades[9] ? '#A50026' :
        d > arrLegendGrades[8]  ? '#D73027' :
        d > arrLegendGrades[7]  ? '#F46D43' :
        d > arrLegendGrades[6]  ? '#FDAE61' :
        d > arrLegendGrades[5]   ? '#FEE08B' :
        d > arrLegendGrades[4]   ? '#D9EF8B' :
        d > arrLegendGrades[3]   ? '#A6D96A' : 
        d > arrLegendGrades[2]   ? '#66BD63' : 
        d > arrLegendGrades[1]   ? '#1A9850' : '#006837';
}

function drawInitialLayer(result) {
    data = result;
    clearLayers();

    dataLayer = L.geoJson(data).addTo(map);
}

function drawLayerSelector() {
    $.ajax({
        type: 'POST',
        url: '/getLayers',
        dataType: 'json',
        contentType: 'application/json; charset=utf-8',
        success: function (result) {
            capes = result;
            map.addControl(new layerSelector(result, {position: 'topright'}));
            var radioLayer = $('input[name=selectedLayer]:radio');
            radioLayer.change(function() {
	       if (this.value.startsWith("$$")) {
                   legend.getContainer().hidden = true;
                   drawSpecialLayers(this.value);
	       } else {
                   legend.getContainer().hidden = false;
                   getDataLayer(this.value);
	       }
            });

            radioLayer[2].click();
        },
        error: function (req, status, error) {
            alert('Unable to get data from server');
        }
    });
}

function clearLayers() {
    if (dataLayer != undefined)
    {
        map.removeLayer(dataLayer);
    }

    if (specialLayer != undefined) {
        map.removeLayer(specialLayer);
    }

    if (marcador != undefined) {
        map.removeLayer(marcador);
        marcador = null;
    }

    if (g != undefined) {
        map.closePopup();
        d3.selectAll("g").remove();
        g.selectAll("path").remove();
        svg.remove();
    }

    map.off("viewreset moveend", reset);


    map.setView([41.69676, 1.88042],8);
}

function getMarkerText(equipament) {
    var result = "";
    result += "Nom: ";
    if (equipament.nom != undefined) result += equipament.nom;
    result += "<br>";
    result += "Adreça: ";
    if (equipament.carrer != undefined) result += equipament.carrer;
    result += "<br>";
    result += "Codi postal: ";
    if (equipament.cp != undefined) result += equipament.cp;
    result += "<br>";
    result += "Municipi: ";
    if (equipament.municipi != undefined) result += equipament.municipi;
    result += "<br>";
    result += "Distancia aproximada: ";
    if (equipament.distancia != undefined && equipament.distancia >= 0) result += numeral(equipament.distancia/1000).format('0.00') + " Km.";

    return result;
}
